<!--
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road, 
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          15-08-2018
//      Created for Project :   FLAME
-->

# **FLAME - Alerts Configuration**

#### **Authors**

|Authors|Organisation|                    
|:---:|:---:|  
|[Nikolay Stanchev](mailto:ns17@it-innovation.soton.ac.uk)|[University of Southampton, IT Innovation Centre](http://www.it-innovation.soton.ac.uk)|


#### Description

This document outlines the configuration of alerts within CLMC. Alerts are configured through a YAML-based
TOSCA-compliant document. This document is passed to the CLMC service, which parses and validates the document. Subsequently, 
the CLMC service creates and activates the alerts within Kapacitor, then registers the HTTP alert handlers specified in the document.


#### TOSCA Alerts Configuration Document

The TOSCA Alerts Configuration Document consists of two main sections - **metadata** and **policies**. Full definitions and
clarification of the structure of the document is given in the following sections. An example of a valid alert configuration
document will look like:

```yaml
tosca_definitions_version: tosca_simple_profile_for_nfv_1_0_0

description: TOSCA Alerts Configuration document

imports:
- flame_clmc_alerts_definitions.yaml

metadata:
  sfc: companyA-VR
  sfci: companyA-VR-premium

topology_template:

  policies:
    - high_latency_policy:
        type: eu.ict-flame.policies.StateChange
        triggers:
          high_latency:
            description: This event triggers when the mean network latency in a given location exceeds a given threshold (in ms).
            event_type: threshold
            metric: network.latency
            condition:
              threshold: 45
              granularity: 120
              aggregation_method: mean
              resource_type:
                location: watershed
              comparison_operator: gt
            action:
              implementation:
                - http://sfemc.flame.eu/notify
                - http://companyA.alert-handler.flame.eu/high-latency
    - low_requests_policy:
        type: eu.ict-flame.policies.StateChange
        triggers:
          low_requests:
            description: |
              This event triggers when the last reported number of requests for a given service function
              falls behind a given threshold.
            event_type: threshold
            metric: storage.requests
            condition:
              threshold: 5
              granularity: 60
              aggregation_method: last
              resource_type:
                sf_package: storage
                sf: storage-users
                location: watershed
              comparison_operator: lt
            action:
              implementation:
                - http://sfemc.flame.eu/notify
                - http://companyA.alert-handler.flame.eu/low-requests
```


##### Metadata

The ***metadata*** section specifies the service function chain ID and the service function chain instance ID, for which this 
alerts configuration relates to. The format is the following:

```yaml
metadata:
    sfc: <sfc_id>
    sfci: <sfc_i_id>
```

##### Policies

The ***policies*** section defines a list of policy nodes, each representing a fully qualified configuration for an
alert within CLMC. The format is the following:

```yaml
topology_template:

    policies:
        - <policy_identifier>:
            type: eu.ict-flame.policies.StateChange
            triggers:
                <event identifier>:
                  description: <optional description for the given event trigger>
                  event_type: <threshold | relative | deadman>
                  metric: <measurement>.<field>
                  condition:
                    threshold: <critical value>
                    granularity: <period in seconds - how often to check whether the event condition is true>
                    aggregation_method: <aggregation function supported by InfluxDB - e.g. 'mean'>
                    resource_type:
                      <CLMC Information Model Tag Name>: <CLMC Information Model Tag Value>
                      <CLMC Information Model Tag Name>: <CLMC Information Model Tag Value>
                      ...
                    comparison_operator: <logical operator to use for comparison, e.g. 'gt', 'lt'
                  action:
                    implementation:
                      - <HTTP Alert Handler URL>
                      - <HTTP Alert Handler URL>
                      ...
        ...
```

##### Definitions

* **policy_identifier** - policy label which should match with a StateChange policy in the TOSCA resource specification document
submitted to the FLAME Orchestrator for consistency.

* **event_identifier** - the name of the event that **MUST** match with the *constraint* event name referenced in the TOSCA resource
specification document submitted to the FLAME Orchestrator.

* **event_type** - the type of TICK Script template to use to create the alert - more information will be provided about 
the different options here, but we assume the most common one will be **threshold**.

* **metric** - the metric to query in InfluxDB, must include measurement name and field name in 
format `<measurement>`.`<field>`

* **threshold** - when using the **threshold** event type, this is the critical value the actual metric is compared to.

* **granularity** - the period in seconds, which instructs Kapacitor how often to query InfluxDB and check whether the
event condition is true.

* **aggregation_method** - the function to use when querying InfluxDB

* **resource_type** - provides context for the given event - key-value pairs for the global tags of the 
CLMC Information Model

* **comparison_operator** - the logical operator to use for comparison - less than, greater than, less than or erual to, etc.

* **implementation** - a list of the URLs of alert handlers to which alert data is sent when the event condition is true.
